@@ -254,6 +254,7 @@ ex_data_table.relop_non_sql <- function(optree,
                                         source_usage = NULL,
                                         source_limit = NULL,
                                         env = parent.frame()) {
+  force(env)
   wrapr::stop_if_dot_args(substitute(list(...)), "rqdatatable::ex_data_table.relop_non_sql")
   if(is.null(source_usage)) {
     source_usage <- columns_used(optree)
@@ -263,19 +264,34 @@ ex_data_table.relop_non_sql <- function(optree,
                      source_limit = source_limit,
                      source_usage = source_usage,
                      env = env)
-  f_df <- optree$f_df
-  if(is.null(f_df)) {
-    stop("rqdatatable::ex_data_table.relop_non_sql df is NULL")
+  if(!is.data.frame(x)) {
+    stop("rqdatatable::ex_data_table.relop_non_sql sub-expression eval was NULL")
+  }
+  res <- NULL
+  f_dt <- optree$f_dt
+  if(!is.null(f_dt)) {
+    # data table specialized impl
+    if(!data.table::is.data.table(x)) {
+      x <- data.table::as.data.table(x)
+    }
+    res <- f_dt(x)
+  } else {
+    # data.frame impl
+    f_df <- optree$f_df
+    if(is.null(f_df)) {
+      stop("rqdatatable::ex_data_table.relop_non_sql df is NULL")
+    }
+    x <- as.data.frame(x)
+    res <- f_df(x)
   }
-  res <- f_df(x)
   if(!is.data.frame(res)) {
     stop("qdataframe::ex_data_table.relop_non_sql f_df did not return a data.frame")
   }
   if(!data.table::is.data.table(res)) {
     res <- data.table::as.data.table(res)
   }
-  if(!isTRUE(all.equal(sort(colnames(res)), sort(optree$columns_produced)))) {
-    stop("qdataframe::ex_data_table.relop_non_sql columns produced did not meet specification")
+  if(!isTRUE(all.equal(sort(colnames(res)), sort(column_names(optree))))) {
+    stop("qdataframe::ex_data_table.relop_non_sql columns produced did not meet declared column specification")
   }
   res
 }
