@@ -11,8 +11,7 @@ order_table <- function(x, orderby, reverse) {
   if(length(reverse)>0) {
     order[orderby %in% reverse] <- -1L
   }
-  data.table::setorderv(x, cols = orderby, order = order)
-  x[]
+  data.table::setorderv(x, cols = orderby, order = order)[]
 }
 
 #' Reorder rows.
@@ -39,6 +38,7 @@ ex_data_table.relop_orderby <- function(optree,
                                         source_usage = NULL,
                                         source_limit = NULL,
                                         env = parent.frame()) {
+  force(env)
   wrapr::stop_if_dot_args(substitute(list(...)), "rqdatatable::ex_data_table.relop_orderby")
   if(is.null(source_usage)) {
     source_usage <- columns_used(optree)
